package com.ekexiu.project.storage.resource.service;

import com.ekexiu.project.storage.resource.dao.StorageDao;
import com.ekexiu.project.storage.resource.po.Storage;
import com.ekexiu.project.storage.system.SessionUser;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.Upload;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.exception.JfwBaseException;
import org.jfw.util.web.fileupload.Item;
import org.jfw.util.web.fileupload.UploadItemIterator;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Created by TT on 2018/9/11.
 */
@Path("/storage")
public class StorageService {
    private static final AtomicInteger FN_IDX = new AtomicInteger(1);
    private File filePath;
    @Autowrie
    private StorageDao storageDao;

    public StorageDao getStorageDao() {
        return storageDao;
    }

    public void setStorageDao(StorageDao storageDao) {
        this.storageDao = storageDao;
    }

    public File getFilePath() {
        return filePath;
    }

    public void setFilePath(File filePath) {
        this.filePath = filePath;
    }

    @Post
    @Path
    public String insert(@JdbcConn(true) Connection con, @LoginUser SessionUser user, @Upload UploadItemIterator it) throws Exception {
        Storage storage = new Storage();
        try {
            while (it.hasNext()) {
                Item item = it.next();
                if (!item.isFormField()) {
                    if (storage.getUrl() != null) {
                        throw new JfwBaseException(-1, "only one file");
                    }
                    String name = normalizeFileName(item.getName());
                    int index = name.lastIndexOf('.');
                    String ext = index >= 0 ? name.substring(index + 1) : "";
                    ext = ext.trim();
                    long fileLen = 0;
                    int len = 0;
                    UploadFile uf = this.buildTargetFile(ext);
                    storage.setFileName(name);
                    storage.setUrl(uf.getUri());
                    File file = uf.getFn();
                    byte[] buf = new byte[8092];
                    OutputStream of = new FileOutputStream(file);
                    try {
                        InputStream in = item.getInputStream();
                        try {
                            while ((len = in.read(buf)) >= 0) {
                                if (len > 0) {
                                    of.write(buf, 0, len);
                                    fileLen += len;
                                }
                            }
                            storage.setSize(fileLen);
                        } finally {
                            in.close();
                        }
                    } finally {
                        of.close();
                    }
                } else {
                    if ("name".equals(item.getFieldName())) {
                        storage.setName(item.getString());
                    } else if ("type".equals(item.getFieldName())) {
                        storage.setType(item.getString());
                    }

                }
            }
        } finally {
            if (it != null)
                it.clean();
        }
        if (storage.getName() == null) {

        }

        String id = StringUtil.buildUUID();
        storage.setHospital(user.getHospital());
        storage.setDepartment(user.getDepartment());
        storage.setOwner(user.getId());
        storage.setId(id);
        storage.setActive(true);
        this.storageDao.insert(con, storage);
        return id;
    }

    @Get
    @Path("/delete")
    @LoginUser
    public void logicDelete(@JdbcConn(true) Connection con, String id) throws SQLException {
        this.storageDao.logicDelete(con, id);
    }


    @Path("/pq")
    @Get
    @LoginUser
    public PageQueryResult<Storage> pageQuery(@JdbcConn Connection con, @Nullable Boolean active, int pageSize, int pageNo) throws SQLException {
        return this.storageDao.pageQuery(con, active, pageSize, pageNo);
    }


    private UploadFile buildTargetFile(String ext) {
        StringBuilder sb = new StringBuilder();
        sb.append(System.currentTimeMillis() / (1000 * 60 * 60 * 24));
        File path = new File(this.filePath, sb.toString());
        if (!path.exists()) {
            synchronized (this) {
                if (!path.mkdirs())
                    throw new RuntimeException("mkdir error[" + path + "]");
                FN_IDX.set(1);
            }
        }
        File file;
        do {
            String fn = FN_IDX.toString();
            if (ext.isEmpty()) {
                file = new File(path, fn);
            } else {
                file = new File(path, fn + "." + ext);
            }
            FN_IDX.incrementAndGet();
        } while (file.exists());
        sb.append("/").append(file.getName());
        UploadFile uf = new UploadFile();
        uf.setFn(file);
        uf.setUri("/" + sb.toString());
        return uf;
    }

    private String normalizeFileName(String fn) {
        int index = fn.indexOf('\\');
        if (index >= 0) {
            fn = fn.substring(fn.lastIndexOf('\\') + 1);
        }
        index = fn.indexOf('/');
        if (index >= 0) {
            fn = fn.substring(fn.lastIndexOf('/') + 1);
        }
        if (fn.length() == 0)
            throw new RuntimeException("invalid filename in Multipart/data request");
        return fn;
    }

    public static class UploadFile {
        private String uri;
        private transient File fn;

        public File getFn() {
            return fn;
        }

        public void setFn(File fn) {
            this.fn = fn;
        }

        public String getUri() {
            return uri;
        }

        public void setUri(String uri) {
            this.uri = uri;
        }
    }

}
