package com.ekexiu.project.storage.system.service;

import com.ekexiu.project.storage.system.SessionUser;
import com.ekexiu.project.storage.system.dao.UserDao;
import com.ekexiu.project.storage.system.po.User;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.method.InvalidSession;
import org.jfw.apt.web.annotation.method.SetSession;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.exception.JfwBaseException;

import java.sql.Connection;
import java.sql.SQLException;

/**
 * Created by TT on 2018/9/11.
 */
@Path("/sys")
public class SysService {

    public static final String DEFAULT_PW_STR = StringUtil.md5("123456");

    private String rootPw = "123456";
    private String rootHospital = "kexiu";
    private String rootDepartment = "kexiu";
    private String rootCreateTime = "20150622";

    public String getRootPw() {
        return rootPw;
    }

    public void setRootPw(String rootPw) {
        this.rootPw = rootPw;
    }

    @Autowrie
    private UserDao userDao;

    public UserDao getUserDao() {
        return userDao;
    }

    public void setUserDao(UserDao userDao) {
        this.userDao = userDao;
    }


    @SetSession("JFW_SESSION_LOGIN_USER=result")
    @Path("/login")
    @Post
    public SessionUser login(@JdbcConn Connection con, String account, String pw) throws SQLException, JfwBaseException {
        if(account.equals("admin")){
            if (pw.equals(rootPw)) {
                SessionUser sessionUser = new SessionUser();
                sessionUser.setName("admin");
                sessionUser.setAccount("admin");
                sessionUser.setHospital(rootHospital);
                sessionUser.setDepartment(rootDepartment);
                sessionUser.setCreateTime(rootCreateTime);
                sessionUser.setActive(true);
                sessionUser.setId("11111111111111111111111111111111");
                return sessionUser;
            }
        }else {
            User user = userDao.login(con, account, StringUtil.md5(pw));
            if (user != null) {
                if (!user.isActive()) {
                    throw new JfwBaseException(-60005, "user is disabled");
                }
                SessionUser sessionUser = new SessionUser();
                sessionUser.setAccount(user.getAccount());
                sessionUser.setActive(user.isActive());
                sessionUser.setId(user.getId());
                sessionUser.setName(user.getName());
                sessionUser.setHospital(user.getHospital());
                sessionUser.setDepartment(user.getDepartment());
                sessionUser.setCreateTime(user.getCreateTime());
                return sessionUser;
            }
        }
        return null;
    }

    @Get
    @Path("/user")
    public SessionUser get(@LoginUser SessionUser user) {
        return user;
    }

    @Get
    @Path("/logout")
    @InvalidSession
    public void logout() {
    }

    @Post
    @Path("/insert")
    @LoginUser
    public String insert(@JdbcConn(true) Connection con, User user) throws SQLException {
        String id = StringUtil.buildUUID();
        user.setId(id);
        user.setActive(true);
        user.setPasswd(DEFAULT_PW_STR);
        this.userDao.insert(con, user);
        return id;
    }

    @Get
    @Path("/check")
    @LoginUser
    public Boolean check(@JdbcConn Connection con,@Nullable Boolean active, String account,@Nullable String id)throws SQLException {
        return this.userDao.queryByAccount(con,active, account,id)==null;
    }

    @Get
    @Path("/qo")
    @LoginUser
    public User query(@JdbcConn Connection con, String id) throws SQLException {
        return this.userDao.query(con, id);
    }

    @Post
    @Path("/update")
    @LoginUser
    public void update(@JdbcConn(true) Connection con, User user) throws SQLException {
        this.userDao.update(con, user);
    }

    @Get
    @Path("/ban")
    @LoginUser
    public void ban(@JdbcConn(true) Connection con, String id) throws SQLException {
        this.userDao.ban(con, id);
    }

    @Get
    @Path("/active")
    @LoginUser
    public void active(@JdbcConn(true) Connection con, String id) throws SQLException {
        this.userDao.active(con, id);
    }

    @Get
    @Path("/pq")
    @LoginUser
    public PageQueryResult<User> pageQuery(@JdbcConn Connection con,@Nullable Boolean active,int pageSize,int pageNo)throws SQLException {
        return this.userDao.pageQuery(con, active, pageSize, pageNo);
    }

    @Post
    @Path("/changePw")
    public void changePw(@JdbcConn(true) Connection con, @LoginUser SessionUser u, String newPw) throws SQLException, JfwBaseException {
        this.userDao.changePw(con, StringUtil.md5(newPw), u.getId());
    }

    @Get
    @Path("/resetPw")
    @LoginUser
    public void resetPw(@JdbcConn(true) Connection con,String id)throws SQLException {
        this.userDao.changePw(con, StringUtil.md5(DEFAULT_PW_STR), id);
    }

}
